@@ -3,7 +3,7 @@
 """
 DOM wrapper around libxml2, specifically the libxml2mod Python extension module.
 
-Copyright (C) 2003, 2004, 2005 Paul Boddie <paul@boddie.org.uk>
+Copyright (C) 2003, 2004, 2005, 2006 Paul Boddie <paul@boddie.org.uk>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@ License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
-__version__ = "0.3.3"
+__version__ = "0.3.5"
 
 from libxml2dom.macrolib import *
 from libxml2dom.macrolib import \
@@ -110,6 +110,11 @@ class NamedNodeMap(object):
     def __str__(self):
         return "{%s}" % ",\n".join(["%s : %s" % (repr(key), repr(value)) for key, value in self.items()])
 
+    def _length(self):
+        return len(self.values())
+
+    length = property(_length)
+
 class NodeList(list):
 
     "A wrapper around node lists."
@@ -117,9 +122,11 @@ class NodeList(list):
     def item(self, index):
         return self[index]
 
-    def length(self):
+    def _length(self):
         return len(self)
 
+    length = property(_length)
+
 # Node classes.
 
 class Node(object):
@@ -367,7 +374,9 @@ class Node(object):
 
     def xpath(self, expr, variables=None, namespaces=None):
         result = Node_xpath(self._node, expr, variables, namespaces)
-        if hasattr(result, "__len__"):
+        if isinstance(result, str):
+            return to_unicode(result)
+        elif hasattr(result, "__len__"):
             return NodeList([get_node(_node, self) for _node in result])
         else:
             return result
