@@ -2,6 +2,22 @@
 
 """
 DOM macros for virtual libxml2mod node methods and properties.
+
+Copyright (C) 2003, 2004, 2005, 2006 Paul Boddie <paul@boddie.org.uk>
+
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 """
 
 import xml.dom
@@ -45,7 +61,24 @@ def _get_prefix_and_localName(name):
         # NOTE: Should raise an exception.
         return None, None
 
+def _find_namespace_for_prefix(node, prefix):
+
+    "Find the namespace definition node in the given 'node' for 'prefix'."
+
+    current = libxml2mod.xmlNodeGetNsDefs(node)
+    while current is not None:
+        if libxml2mod.name(current) == prefix:
+            return current
+        current = libxml2mod.next(current)
+    return None
+
 def _find_namespace(node, ns, prefix):
+
+    """
+    Find the namespace definition node in the given 'node' for the given 'ns'
+    and 'prefix'.
+    """
+
     new_ns = None
     current = libxml2mod.xmlNodeGetNsDefs(node)
     while current is not None:
@@ -60,24 +93,50 @@ def _find_namespace(node, ns, prefix):
     return new_ns
 
 def _check_namespace(current, ns, prefix):
-    current_ns = libxml2mod.xmlNodeGetContent(current)
+
+    "Check the 'current' namespace definition node against 'ns' and 'prefix'."
+
+    current_ns = get_ns(current)
     current_prefix = libxml2mod.name(current)
-    if ns == current_ns and prefix == current_prefix:
+    if ns == current_ns and (prefix is None or prefix == current_prefix):
         return 1
     else:
         return 0
 
 def _make_namespace(node, ns, prefix, set_default=0):
+
+    """
+    Make a new namespace definition node within the given 'node' for 'ns',
+    'prefix', setting the default namespace on 'node' when 'prefix' is None and
+    'set_default' is set to a true value (unlike the default value for that
+    parameter).
+    """
+
     if prefix is not None or set_default:
         new_ns = libxml2mod.xmlNewNs(node, ns, prefix)
     else:
         new_ns = None
     return new_ns
 
+def _get_invented_prefix(node, ns):
+    current = libxml2mod.xmlNodeGetNsDefs(node)
+    prefixes = []
+    while current is not None:
+        current_prefix = libxml2mod.name(current)
+        prefixes.append(current_prefix)
+        current = libxml2mod.next(current)
+    i = 0
+    while 1:
+        prefix = "NS%d" % i
+        if prefix not in prefixes:
+            return prefix
+        i += 1
+
 _nodeTypes = {
     "attribute" : xml.dom.Node.ATTRIBUTE_NODE,
     "comment" : xml.dom.Node.COMMENT_NODE,
     "document_xml" : xml.dom.Node.DOCUMENT_NODE,
+    "document_html" : xml.dom.Node.DOCUMENT_NODE,
     "doctype" : xml.dom.Node.DOCUMENT_TYPE_NODE,
     "dtd" : xml.dom.Node.DOCUMENT_TYPE_NODE, # NOTE: Needs verifying.
     "element" : xml.dom.Node.ELEMENT_NODE,
@@ -113,14 +172,27 @@ def Node_childNodes(node):
 
 def Node_attributes(node):
     attributes = {}
-    node = libxml2mod.properties(node)
-    while node is not None:
-        ns = libxml2mod.xmlNodeGetNs(node)
+
+    # Include normal attributes.
+
+    current = libxml2mod.properties(node)
+    while current is not None:
+        ns = libxml2mod.xmlNodeGetNs(current)
         if ns is not None:
-            attributes[(get_ns(ns), libxml2mod.name(node))] = node
+            attributes[(get_ns(ns), libxml2mod.name(current))] = current
         else:
-            attributes[(None, libxml2mod.name(node))] = node
-        node = libxml2mod.next(node)
+            attributes[(None, libxml2mod.name(current))] = current
+        current = libxml2mod.next(current)
+
+    # Include xmlns attributes.
+
+    #current = libxml2mod.xmlNodeGetNsDefs(node)
+    #while current is not None:
+    #    ns = get_ns(current)
+    #    prefix = libxml2mod.name(current)
+    #    attributes[(xml.dom.XMLNS_NAMESPACE, "xmlns:" + prefix)] = ns # NOTE: Need a real node here.
+    #    current = libxml2mod.next(current)
+
     return attributes
 
 def Node_namespaceURI(node):
@@ -189,13 +261,21 @@ def Node_doctype(node):
     return libxml2mod.xmlGetIntSubset(node)
 
 def Node_hasAttributeNS(node, ns, localName):
-    return Node_getAttributeNS(node, ns, localName) is not None
+    return Node_getAttributeNS(node, ns, localName) is not None or \
+        _find_namespace(node, ns, localName) is not None
 
 def Node_hasAttribute(node, name):
     return Node_getAttribute(node, name) is not None
 
 def Node_getAttributeNS(node, ns, localName):
-    return to_unicode(libxml2mod.xmlGetNsProp(node, localName, ns))
+    if ns == xml.dom.XMLNS_NAMESPACE:
+        ns_def = _find_namespace_for_prefix(node, localName)
+        if ns_def is not None:
+            return get_ns(ns_def)
+        else:
+            return None
+    else:
+        return to_unicode(libxml2mod.xmlGetNsProp(node, localName, ns))
 
 def Node_getAttribute(node, name):
     return to_unicode(libxml2mod.xmlGetProp(node, name))
@@ -222,8 +302,13 @@ def Node_setAttributeNS(node, ns, name, value):
     # For non-xmlns attributes, we find or make a namespace declaration and then
     # set an attribute.
     elif ns is not None:
+        # Look for a suitable namespace.
         new_ns = _find_namespace(node, ns, prefix)
+        # Create a declaration if no suitable one was found.
         if new_ns is None:
+            # Invent a prefix for unprefixed attributes with namespaces.
+            if prefix is None:
+                prefix = _get_invented_prefix(node, ns)
             new_ns = _make_namespace(node, ns, prefix, set_default=0)
         libxml2mod.xmlSetNsProp(node, new_ns, localName, value)
     else:
